#!/usr/bin/env bash

setup_file(){
  load "$PWD/tests/integration/helpers/common"
  _common_setup
  _add_server
  _new_app
}

teardown_file(){
  _rm_app
  _rm_server
  _reset_recipe
}

setup(){
  load "$PWD/tests/integration/helpers/common"
  _common_setup
}

teardown(){
  _reset_recipe
  _reset_tags
  _undeploy_app
}

@test "validate app argument" {
  run $ABRA app cmd
  assert_failure

  run $ABRA app cmd DOESNTEXIST
  assert_failure
}

@test "retrieve recipe if missing" {
  run rm -rf "$ABRA_DIR/recipes/$TEST_RECIPE"
  assert_success
  assert_not_exists "$ABRA_DIR/recipes/$TEST_RECIPE"

  run $ABRA app cmd --local "$TEST_APP_DOMAIN" test_cmd
  assert_success
  assert_output --partial 'baz'

  assert_exists "$ABRA_DIR/recipes/$TEST_RECIPE"
}

@test "bail if unstaged changes and no --chaos" {
  run bash -c "echo foo >> $ABRA_DIR/recipes/$TEST_RECIPE/foo"
  assert_success
  assert_exists "$ABRA_DIR/recipes/$TEST_RECIPE/foo"

  run $ABRA app cmd --local "$TEST_APP_DOMAIN" test_cmd
  assert_failure
  assert_output --partial 'locally unstaged changes'

  assert_exists "$ABRA_DIR/recipes/$TEST_RECIPE/foo"
  assert_equal "$(_git_status)" "?? foo"

  run rm -rf "$ABRA_DIR/recipes/$TEST_RECIPE/foo"
  assert_not_exists "$ABRA_DIR/recipes/$TEST_RECIPE/foo"
}

@test "do not bail if unstaged changes and --chaos" {
  run bash -c "echo foo >> $ABRA_DIR/recipes/$TEST_RECIPE/foo"
  assert_success
  assert_exists "$ABRA_DIR/recipes/$TEST_RECIPE/foo"

  run $ABRA app cmd --local --chaos "$TEST_APP_DOMAIN" test_cmd
  assert_success
  assert_output --partial 'baz'

  assert_exists "$ABRA_DIR/recipes/$TEST_RECIPE/foo"
  assert_equal "$(_git_status)" "?? foo"

  run rm -rf "$ABRA_DIR/recipes/$TEST_RECIPE/foo"
  assert_not_exists "$ABRA_DIR/recipes/$TEST_RECIPE/foo"
}

@test "ensure recipe up to date if no --offline" {
  wantHash=$(_get_n_hash 3)

  run git -C "$ABRA_DIR/recipes/$TEST_RECIPE" reset --hard HEAD~3
  assert_success

  assert_equal $(_get_current_hash) "$wantHash"

  run $ABRA app cmd --local "$TEST_APP_DOMAIN" test_cmd
  assert_success
  assert_output --partial 'baz'

  assert_equal $(_get_head_hash) $(_get_current_hash)
}

@test "ensure recipe not up to date if --offline" {
  _ensure_env_version "0.1.0+1.20.0"
  latestRelease=$(_latest_release)

  run git -C "$ABRA_DIR/recipes/$TEST_RECIPE" tag -d "$latestRelease"
  assert_success

  run $ABRA app cmd --local --offline "$TEST_APP_DOMAIN" test_cmd
  assert_success
  assert_output --partial 'baz'

  run git -C "$ABRA_DIR/recipes/$TEST_RECIPE" tag -l
  refute_output --partial "$latestRelease"
}

@test "error if missing arguments without passing --local" {
  run $ABRA app cmd "$TEST_APP_DOMAIN"
  assert_failure
}

@test "error if missing arguments when passing --local" {
  run $ABRA app cmd --local "$TEST_APP_DOMAIN"
  assert_failure
}

@test "cannot use --local and --user at same time" {
  run $ABRA app cmd --local --user root "$TEST_APP_DOMAIN" test_cmd
  assert_failure
  assert_output --partial 'cannot use --local & --user together'
}

@test "error if missing abra.sh" {
  run rm -rf "$ABRA_DIR/recipes/$TEST_RECIPE/abra.sh"
  assert_success

  run $ABRA app cmd --local --chaos "$TEST_APP_DOMAIN" test_cmd
  assert_failure
  assert_output --partial "$ABRA_DIR/recipes/$TEST_RECIPE/abra.sh does not exist"

  _checkout_recipe
}

@test "error if missing command" {
  run $ABRA app cmd --local "$TEST_APP_DOMAIN" doesnt_exist
  assert_failure
  assert_output --partial "doesn't have a doesnt_exist function"
}

@test "run --local command" {
  run $ABRA app cmd --local "$TEST_APP_DOMAIN" test_cmd
  assert_success
  assert_output --partial 'baz'
}

@test "run command with single arg" {
  run $ABRA app cmd --local "$TEST_APP_DOMAIN" test_cmd_arg -- bing
  assert_success
  assert_output --partial 'bing'
}

@test "run command with several args" {
  run $ABRA app cmd --local "$TEST_APP_DOMAIN" test_cmd_args bong bang
  assert_success
  assert_output --partial 'bong bang'
}

# bats test_tags=slow
@test "run command on service" {
  _deploy_app

  run $ABRA app cmd "$TEST_APP_DOMAIN" app test_cmd
  assert_success
  assert_output --partial 'baz'
}

# bats test_tags=slow
@test "respects env version" {
  tagHash=$(_get_tag_hash "0.1.0+1.20.0")

  run $ABRA app deploy "$TEST_APP_DOMAIN" "0.1.0+1.20.0" --no-input
  assert_success

  run grep -q "TYPE=$TEST_RECIPE:0.1.0+1.20.0" \
    "$ABRA_DIR/servers/$TEST_SERVER/$TEST_APP_DOMAIN.env"
  assert_success

  run $ABRA app cmd "$TEST_APP_DOMAIN" app test_cmd
  assert_success
  assert_output --partial 'baz'

  assert_equal $(_get_current_hash) "$tagHash"
}

# bats test_tags=slow
@test "error if missing service" {
  _deploy_app

  run $ABRA app cmd "$TEST_APP_DOMAIN" doesnt_exist test_cmd
  assert_failure
  assert_output --partial 'no service doesnt_exist'
}
